//*********************************************************************
// Example5Tangent.java
//
// Uses two versions of a function calculating tg(x).
//*********************************************************************

public class Example5Tangent
{
   public static void main(String[] args)
   {
      long time;
      System.out.println("Tangent of 30 degrees: ");
      time = System.currentTimeMillis();
      System.out.println("By method 1: " +  Tangent.tg_1(Math.PI/6) +
                         " (time - " + (System.currentTimeMillis() - time) + ")");
      time = System.currentTimeMillis();
      System.out.println("By method 2: " +  Tangent.tg_2(Math.PI/6) +
                         " (time - " + (System.currentTimeMillis() - time) + ")");
      System.out.println("Using Math.tan: " + Math.tan(Math.PI/6));
      System.out.println("Theoretical: " + Math.sqrt(1./3));

      System.out.println("\nTangent of 480 degrees: ");
      time = System.currentTimeMillis();
      System.out.println("By method 1: " +  Tangent.tg_1(8 * Math.PI/3) +
                         " (time - " + (System.currentTimeMillis() - time) + ")");
      time = System.currentTimeMillis();
      System.out.println("By method 2: " +  Tangent.tg_2(8 * Math.PI/3) +
                         " (time - " + (System.currentTimeMillis() - time) + ")");
      System.out.println("Using Math.tan: " + Math.tan(8 * Math.PI/3));
      System.out.println("Theoretical: " + -Math.sqrt(3));
   }// method main
}// class Example5Tangent

class Tangent
{
   //=====================================================================
   //  Contains two versions of a function calculating tg(x). Both
   //  versions are recursive, using the estimate tg(x)~x for x~0
   //  and the formula tg(x)=2tg(x/2)/(1-tg^2(x/2)) otherwise.
   //=====================================================================
   public static double tg_1(double x)
   {
      double tg;
      if (Math.abs(x) < 1E-4)
         tg = x;
      else
         tg = 2 * tg_1(x/2) / (1 - tg_1(x/2) * tg_1(x/2));
         return tg;
   }// method tg_1

   public static double tg_2(double x)
   {
      // The same calculations are performed as in the previous method.
      // However, instead of three recursive calls there is only one, which
      // makes a very big difference in terms of computation time.
      double tg, tgOfHalfAngle;
      if (Math.abs(x) < 1E-4)
         tg = x;
      else
      {
         tgOfHalfAngle = tg_2(x/2);
         tg = 2 * tgOfHalfAngle / (1 - tgOfHalfAngle * tgOfHalfAngle);
      }
         return tg;
   }// method tg_2
}// class Tangent
